-- $Id: adv_consistency.sql 2923 2012-01-07 09:58:29Z coelho $

-- --------------------------------------------------
-- Consistency Advices
-- --------------------------------------------------

--
-- Source Table & Constraint in different schemas
--
-- (seems impossible to create)
-- VIEW
CREATE TABLE salix.ca_cross_schema_src_constraints AS

SELECT
  tc.constraint_schema AS ConstraintSchema,
  tc.constraint_name AS ConstraintName,
  tc.table_schema AS SourceTableSchema,
  tc.table_name AS SourceTableName,
  CONCAT('constraint ', tc.constraint_schema, '.', tc.constraint_name,
  	 ' from table ', tc.table_schema, '.', tc.table_name) AS explanation
FROM TABLE_CONSTRAINTS AS tc
WHERE tc.constraint_schema <> tc.table_schema
  AND tc.constraint_schema <> ALL (SELECT name FROM salix.excluded_schemas);

-- ALTER TABLE salix.ca_cross_schema_src_constraints
--   ADD PRIMARY KEY
--     (ConstraintSchema,ConstraintName,SourceTableSchema,SourceTableName);

--
-- Destination Table & Constraint in different schemas
--
-- VIEW
CREATE TABLE salix.ca_cross_schema_dst_constraints AS

SELECT
  tc.constraint_schema AS SourceConstraintSchema,
  tc.constraint_name AS SourceConstraintName,
  tc.table_schema AS SourceTableSchema,
  tc.table_name AS SourceTableName,
  ftc.constraint_schema AS DestinationConstraintSchema,
  ftc.constraint_name AS DestinationConstraintName,
  ftc.table_schema AS DestinationTableSchema,
  ftc.table_name AS DestinationTableName,
  CONCAT('constraint ', tc.constraint_schema, '.', tc.constraint_name,
  	 ' on table ', tc.table_schema, '.', tc.table_name) AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN REFERENTIAL_CONSTRAINTS AS rc
  ON tc.constraint_schema=rc.constraint_schema
     AND tc.constraint_name=rc.constraint_name
JOIN TABLE_CONSTRAINTS AS ftc
  ON rc.unique_constraint_schema=ftc.constraint_schema
     AND rc.unique_constraint_name=ftc.constraint_name
WHERE tc.constraint_type = 'FOREIGN KEY'
  AND (tc.constraint_schema <> ftc.constraint_schema
    OR tc.table_schema <> ftc.table_schema)
  AND tc.constraint_schema <> ALL (SELECT name FROM salix.excluded_schemas);

-- ALTER TABLE salix.ca_cross_schema_dst_constraints
--   ADD PRIMARY KEY (SourceConstraintSchema,SourceConstraintName,
--                    SourceTableSchema,SourceTableName);

--
-- Tables Linked But in Different Schemas
--
-- VIEW
CREATE TABLE salix.ca_cross_schema_tables AS

SELECT
  t.table_schema AS SourceTableSchema,
  t.table_name AS SourceTableName,
  t2.table_schema AS DestinationTableSchema,
  t2.table_name AS DestinationTableName,
  CONCAT('table ', t.table_schema, '.', t.table_name,
  	 ' to ', t2.table_schema, '.', t2.table_name) AS explanation
FROM TABLES AS t
JOIN TABLE_CONSTRAINTS AS tc
  ON t.table_schema=tc.table_schema AND t.table_name=tc.table_name
JOIN REFERENTIAL_CONSTRAINTS AS rc
  ON tc.constraint_schema=rc.constraint_schema
     AND tc.constraint_name=rc.constraint_name
JOIN TABLE_CONSTRAINTS AS ftc
  ON rc.unique_constraint_schema=ftc.constraint_schema
     AND rc.unique_constraint_name=ftc.constraint_name
JOIN TABLES AS t2
  ON t2.table_schema=ftc.table_schema AND t2.table_name=ftc.table_name
WHERE tc.constraint_type = 'FOREIGN KEY'
  AND t.table_schema <> t2.table_schema
  AND t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas);

-- ALTER TABLE salix.ca_cross_schema_tables
--   ADD PRIMARY KEY (SourceTableSchema,SourceTableName,
--                    DestinationTableSchema,DestinationTableName);
