-- $Id: adv_description.sql 2923 2012-01-07 09:58:29Z coelho $
--
-- Fill Advice Information Tables:
-- Syntax is MySQL/PostgreSQL/SQLite compatible
--

INSERT INTO salix.advice_category(cid, description) VALUES (1, 'design');
INSERT INTO salix.advice_category(cid, description) VALUES (2, 'consistency');
INSERT INTO salix.advice_category(cid, description) VALUES (3, 'version');
INSERT INTO salix.advice_category(cid, description) VALUES (4, 'style');
INSERT INTO salix.advice_category(cid, description) VALUES (5, 'system');

INSERT INTO salix.advice_severity(sid, description) VALUES (1, 'information');
INSERT INTO salix.advice_severity(sid, description) VALUES (2, 'notice');
INSERT INTO salix.advice_severity(sid, description) VALUES (3, 'warning');
INSERT INTO salix.advice_severity(sid, description) VALUES (4, 'error');

INSERT INTO salix.advice_level(lid, description) VALUES (1, 'schema');
INSERT INTO salix.advice_level(lid, description) VALUES (2, 'table');
INSERT INTO salix.advice_level(lid, description) VALUES (3, 'attribute');
INSERT INTO salix.advice_level(lid, description) VALUES (4, 'user');
INSERT INTO salix.advice_level(lid, description) VALUES (5, 'database');
-- constraints? global?

--
-- Design Advices
--

-- 4 error => possible bug
-- Tables Without Primary Key
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_tables_wo_primary_key_but_unique', 1, 3, 2,
 'Tables without PK but with Unique',
 'All tables should have a primary key to be consistant with the set theory. A unique constraint may be promoted as the primary key.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_tables_wo_p_key_and_unique', 1, 4, 2,
 'Tables without PK nor Unique',
 'All tuples must be uniquely defined to be consistant with the set theory. There is no unique subset of attribute which can be promoted as a PK.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_tables_w_composite_p_key',	1, 2, 2,
 'Tables with Composite PK',
 'A simple primary key, without specific semantics, is usually a better design, and references through foreign keys are simpler.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_tables_w_composite_f_key',	1, 3, 2,
 'Composite Foreign Key',
 'As for primary keys, simple foreign keys are usually better design, and make updates easier.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_non_integer_p_key', 1, 2, 2,
 'Non-integer Primary Key',
 'Having integer primary keys without specific application semantics make updates easier.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_int_pk_but_no_other_key', 1, 3, 2,
 'Integer PK but no other key',
 'A simple integer primary key suggests that some other key must exist in the table.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_unique_nullable_attribute', 1, 3, 3,
 'Unique nullable attributes',
 'A unique nullable attribute may be a bad design if NULL does not have a particular semantic.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_nullable_attribute_rate_high', 1, 1, 1,
 'Nullable attribute rate in 20-50%',
 'Information: Most of the time, attributes should be NOT NULL. Too high a rate of nullable attribute may reveal that some fields are lacking a NOT NULL.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_nullable_attribute_rate_quite_high', 1, 2, 1,
 'Nullable attribute rate in 50-80%',
 'Notice: Most of the time, attributes should be NOT NULL. Too high a rate of nullable attribute may reveal that some fields are lacking a NOT NULL.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_nullable_attribute_rate_too_high', 1, 3, 1,
 'Nullable attribute rate over 80%',
 'Warning: Most of the time, attributes should be NOT NULL. Too high a rate of nullable attribute may reveal that some fields are lacking a NOT NULL.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_f_key_type_dont_match', 2, 4, 2, -- should be 3?
 'FK type mismatch',
 'A Foreign Key should have matching referencing and referenced types.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_f_key_match_b_diff_sizes',	2, 4, 2, -- idem
 'FK length mismatch',
 'A Foreign Key should have matching referencing and referenced type sizes.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_large_p_keys_referenced_by_f_key',	1, 3, 2, -- idem
 'Large PK referenced by a FK',
 'Having large primary keys referenced by a foreign key may reveal data duplication, as the primary key is likely to contain relevant information.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_f_key_not_ref_p_key', 1, 3, 2, -- idem
 'FK not referencing a PK',
 'A Foreign Key should rather reference a Primary Key.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_isolated_table', 1, 3, 2,
 'Isolated Tables',
 'In a database design, tables are usually linked together.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_same_name_diff_types', 4, 3, 3,
 'Homonymous heterogeneous attributes',
 'Better avoid using the same attribute name with different types on different tables in the same application, as it may confuse the developer.',
 NULL);
 -- if MyISAM, the next one is always triggered
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_schema_wo_FK', 1, 4, 1,
 'Schema without any FK',
 'Why use a relational database if data are not related at all? Well, that might happen...',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_table_names_mixed_style', 4, 2, 1,
 'Mixed table name styles',
 'Better use homogeneous table names.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_names_mixed_style', 4, 2, 2,
 'Mixed attribute name styles',
 'Better use homogeneous attribute names.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_table_identifiers_length_too_short', 4, 3, 2,
 'Table name length too short',
 'A table name with less than 2 characters is really too short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES('da_table_identifiers_length_quite_short', 4, 2, 2,
 'Table name length quite short',
 'A table name with 3 characters is quite short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_table_identifiers_length_short', 4, 1, 2,
 'Table name length short',
 'A table name with 4 characters is short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_identifiers_length_too_short', 4, 3, 3,
 'Attribute name length too short',
 'An attribute name with 1 character is really too short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_identifiers_length_quite_short', 4, 2, 3,
 'Attribute name length quite short',
 'An attribute name of 2 characters is quite short (but "id" and "pk").',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_identifiers_length_short', 4, 1, 3,
 'Attribute name length short',
 'An attribute name with 3 characters is short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_count_per_table_too_many', 1, 3, 2,
 'Attribute count per table over 40',
 'Having so many attributes in the same table may reveal the need for additional relations.',
  NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES('da_attribute_count_per_table_many', 1, 2, 2,
 'Attribute count per table over 20',
 'Having many attributes in the same table may suggest the need for additional relations.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES('da_attribute_pseudo_null_default', 1, 3, 3,
 'Attribute has a pseudo ''NULL'' text default',
 'Possibly the NULL value was intended instead of the ''NULL'' text.',
 NULL);
-- this one cannot be triggered by the tests, as we ensure one db per project
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('ca_cross_schema_src_constraints', 2, 3, 2,
 'Source table and constraint in different schemas',
 'A constraint and its source table should be in the same schema.',
 NULL);
-- idem
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('ca_cross_schema_indexes', 2, 3, 2,
 'Table and index in different schemas',
 'An index and its table should be in the same schema.',
 NULL);
-- idem
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('ca_cross_schema_dst_constraints', 2, 3, 2,
 'Destination table and FK in different schemas',
 'A constraint and its destination table are usually in the same schema.',
 NULL);
-- idem
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('ca_cross_schema_tables', 2, 2, 2,
 'Tables linked but in different schemas',
 'Linked tables are usually in the same schema.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('dbva_database_is_mysql', 3, 2, 5,
 'MySQL is used',
 'MySQL lacks important features of the SQL standard, including missing set operators.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('dbva_table_engine_is_unsafe', 3, 1, 2,
 'Unsafe backend engine used on table',
 'An unsafe backend (e.g. MyISAM) lacks referential integrity, transaction support, and is not crash safe.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('dbva_schema_engine_is_unsafe', 3, 3, 1,
 'Unsafe backend engine used in schema',
 'An unsafe backend (e.g. MyISAM) used at least once lacks referential integrity, transaction support, and is not crash safe.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('dbva_engine_inconsistency', 3, 4, 1,
 'Backend engine inconsistency',
 'Different backends are used in the same database. It may be legitimate to do so if a particular feature of one backend is needed, for instance full text indexes.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('sa_trivial_password_user', 5, 3, 4,
 'User with weak password',
 'User with empty or username password.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('sa_trivial_password_superuser', 5, 4, 4,
 'SuperUser with weak password',
 'SuperUser with empty or username password.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('sa_redundant_indexes', 5, 3, 2,
 'Redundant indexes',
 'Redundant indexes are costly to maintain.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('sa_table_without_index', 5, 2, 2,
 'Table without index',
 'Not a single index on a table.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('sa_redundant_foreign_keys', 5, 3, 2,
 'Redundant FK',
 'Redundant Foreign Keys are costly to maintain.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_schema_identifiers_length_too_short', 4, 3, 1,
 'Schema name length too short',
 'A schema name with less than 3 characters is really too short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES('da_schema_identifiers_length_quite_short', 4, 2, 1,
 'Schema name length quite short',
 'A schema name with 4 characters is quite short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_schema_identifiers_length_short', 4, 1, 1,
 'Schema name length short',
 'A schema name with 5 characters is short.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_attribute_with_table_prefix', 4, 2, 3,
 'Attribute named after its table',
 'An attribute contains the name of its table, which is redundant.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_no_attribute_table', 1, 4, 2,
 'No attribute in table',
 'There must be something in a table.',
 NULL);
INSERT INTO salix.advice_info
  (view_name, cid, sid, lid, title, abstract, description)
VALUES ('da_single_attribute_table', 1, 2, 2,
 'Table with a single attribute',
 'Possibly some more attributes are needed to have a semantic.',
 NULL);
