-- $Id: adv_design.sql 2923 2012-01-07 09:58:29Z coelho $

-- ------------------------------------------------------------
-- Design Advices
-- ------------------------------------------------------------

--
-- Tables Without Primary Key
--
CREATE TABLE salix.da_tables_wo_primary_key AS
SELECT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  'table ' || t.table_schema || '.' || t.table_name ||
    ' without PK' AS explanation
FROM TABLES AS t
WHERE t.table_type = 'BASE TABLE'
  AND ((t.table_schema,t.table_name) NOT IN	-- !! no EXCEPT in MySQL
     (SELECT DISTINCT
       tc.table_schema AS TSchema,
       tc.table_name AS TName
      FROM TABLE_CONSTRAINTS AS tc
      WHERE tc.constraint_type = 'PRIMARY KEY'))
  AND t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas);

-- ALTER TABLE salix.da_tables_wo_primary_key
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Tables Without PK And Without Unique Constraint
--
-- VIEW
CREATE TABLE salix.da_tables_wo_p_key_and_unique AS
SELECT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
WHERE t.table_type = 'BASE TABLE'
  AND ((t.table_schema,t.table_name) NOT IN	-- !! no EXCEPT in MySQL
(SELECT DISTINCT
  tc.table_schema AS TSchema,
  tc.table_name AS TName
FROM TABLE_CONSTRAINTS AS tc
WHERE tc.constraint_type = 'PRIMARY KEY'
   OR tc.constraint_type = 'UNIQUE'))
  AND (t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- ALTER TABLE salix.da_tables_wo_p_key_and_unique
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Tables Without Primary Key but With Unique
--
CREATE TABLE salix.da_tables_wo_primary_key_but_unique AS
SELECT TSchema, TName,
  CONCAT('table ', TSchema, '.', TName) AS explanation
FROM salix.da_tables_wo_primary_key
WHERE (Tschema, TName)
  NOT IN (SELECT Tschema, TName FROM salix.da_tables_wo_p_key_and_unique);

-- ALTER TABLE salix.da_tables_wo_primary_key_but_unique
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Tables With Composite Primary Key
--
-- VIEW
CREATE TABLE salix.da_tables_w_composite_p_key AS
SELECT
  tc.table_schema AS TSchema,
  tc.table_name AS TName,
  COUNT(*) AS Nb,
  CONCAT('table ', tc.table_schema, '.', tc.table_name,
    ' with ', salix.textcast(COUNT(*)), '-composite PK') AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema)
  AND (tc.constraint_name=kcu.constraint_name)
  AND (tc.table_schema=kcu.table_schema)
  AND (tc.table_name=kcu.table_name)
WHERE tc.constraint_type = 'PRIMARY KEY'
  AND (tc.table_schema <> ALL (SELECT name FROM salix.excluded_schemas))
GROUP BY tc.table_schema,tc.table_name
HAVING COUNT(*) > 1;

-- ALTER TABLE salix.da_tables_w_composite_p_key
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Composite Foreign Key
--
-- VIEW
CREATE TABLE salix.da_tables_w_composite_f_key AS
SELECT
  tc.table_schema AS TSchema,
  tc.table_name AS TName,
  tc.constraint_name AS CName,
  COUNT(*) AS Nb,
  CONCAT('table ', tc.table_schema, '.', tc.table_name,
    ' with ', salix.textcast(COUNT(*)), '-composite FK ', tc.constraint_name)
     AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema)
  AND (tc.constraint_name=kcu.constraint_name)
  AND (tc.table_schema=kcu.table_schema)
  AND (tc.table_name=kcu.table_name)
WHERE tc.constraint_type = 'FOREIGN KEY'
  AND (tc.table_schema <> ALL (SELECT name FROM salix.excluded_schemas))
GROUP BY tc.table_schema,tc.table_name,tc.constraint_name
HAVING COUNT(*) > 1;

--
-- ah ah!
--
CREATE TABLE salix.da_no_attribute_table AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  CONCAT('table ', c.table_schema, '.', c.table_name) AS explanation
FROM TABLES AS t
LEFT JOIN COLUMNS AS c ON (t.table_schema = c.table_schema AND
                           t.table_name = c.table_name)
WHERE c.column_name IS NULL
  AND t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas);

--
-- from SchemaSpy
--
CREATE TABLE salix.da_single_attribute_table AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  CONCAT('table ', c.table_schema, '.', c.table_name) AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
GROUP BY TSchema, TName
HAVING COUNT(*)=1;

-- ALTER TABLE salix.da_tables_w_composite_f_key
--   ADD PRIMARY KEY (TSchema,TName,CName);

--
-- Non Integer Primary Key
--
-- VIEW
CREATE TABLE salix.da_non_integer_p_key AS
SELECT
  tc.table_schema AS TSchema,
  tc.table_name AS TName,
  kcu.column_name AS CName,
  c.data_type AS CType,
  CONCAT('table ', tc.table_schema, '.', tc.table_name) AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema)
  AND (tc.constraint_name=kcu.constraint_name)
  AND (tc.table_schema=kcu.table_schema)
  AND (tc.table_name=kcu.table_name)
JOIN COLUMNS AS c
  ON (kcu.table_schema = c.table_schema)
  AND (kcu.table_name=c.table_name)
  AND (kcu.column_name=c.column_name)
WHERE tc.constraint_type = 'PRIMARY KEY'
  AND c.data_type <> ALL (SELECT type_name FROM salix.primary_key_types)
  AND tc.table_schema <> ALL (SELECT name FROM salix.excluded_schemas);

-- ALTER TABLE salix.da_non_integer_p_key
--   ADD PRIMARY KEY (TSchema,TName,CName);

-- tables with a simple int key
CREATE TABLE salix.in_simple_pkey AS
SELECT DISTINCT
  tc.table_schema AS TSchema,
  tc.table_name AS TName
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema)
  AND (tc.constraint_name=kcu.constraint_name)
  AND (tc.table_schema=kcu.table_schema)
  AND (tc.table_name=kcu.table_name)
JOIN COLUMNS AS c
  ON (kcu.table_schema = c.table_schema)
  AND (kcu.table_name=c.table_name)
  AND (kcu.column_name=c.column_name)
WHERE tc.constraint_type = 'PRIMARY KEY'
  -- int type
  AND c.data_type = ANY (SELECT type_name FROM salix.primary_key_types)
  AND tc.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  -- alas, EXCEPT is not implemented by MySQL
  -- table has no composite PK
  AND (tc.table_schema, tc.table_name)
    NOT IN (SELECT Tschema, Tname FROM salix.da_tables_w_composite_p_key);

--
-- Table with Integer Primary Key but no other Key...
--
CREATE TABLE salix.da_int_pk_but_no_other_key AS
SELECT DISTINCT spk.TSchema, spk.TName,
  CONCAT('table ', spk.TSchema, '.', spk.TName) AS explanation
FROM salix.in_simple_pkey AS spk
  -- alas, EXCEPT is not implemented by MySQL
  -- table has no unique
WHERE (TSchema, TName) NOT IN
   (SELECT DISTINCT tc.table_schema, tc.table_name
    FROM TABLE_CONSTRAINTS AS tc
    WHERE tc.constraint_type = 'UNIQUE');

--
-- Unique Nullable Attributes
--
-- VIEW
CREATE TABLE salix.da_unique_nullable_attribute AS
SELECT
  tc.table_schema AS TSchema,
  tc.table_name AS TName,
  kcu.column_name AS CName,
  tc.constraint_name AS ConstraintName,
  CONCAT('attribute ', kcu.column_name,
  ' on table ', tc.table_schema, '.', tc.table_name) AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema)
  AND (tc.constraint_name=kcu.constraint_name)
  AND (tc.table_schema=kcu.table_schema)
  AND (tc.table_name=kcu.table_name)
JOIN COLUMNS AS c
  ON (kcu.table_schema=c.table_schema)
  AND (kcu.table_name=c.table_name)
  AND (kcu.column_name=c.column_name)
WHERE tc.constraint_type = 'UNIQUE'
  AND (c.is_nullable = 'YES')
  AND (tc.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- ALTER TABLE salix.da_unique_nullable_attribute
--   ADD PRIMARY KEY (TSchema,TName,CName,ConstraintName);

--
-- Nullable Attributes
--

--
-- from SchemaSpy anomalies:
--
-- pseudo 'NULL' default (null as a string)
--
CREATE TABLE salix.da_attribute_pseudo_null_default AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  c.column_name AS CName,
  CONCAT('attribute ', c.column_name,
  ' on table ', c.table_schema, '.', c.table_name) AS explanation
FROM COLUMNS AS c
WHERE (c.column_default LIKE '%''NULL''%' OR
       c.column_default LIKE '%''null''%' OR
       c.column_default LIKE '%''Null''%')
  AND (c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- in PostgreSQL, NN is also a table_constraints, but it is not in MySQL.
--
-- VIEW
CREATE TABLE salix.in_nullable_attribute AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  c.column_name AS CName
FROM COLUMNS AS c
WHERE c.is_nullable = 'YES'
  AND (c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- ALTER TABLE salix.in_nullable_attribute
--   ADD PRIMARY KEY (TSchema,TName,CName);

--
-- Nullable Attribute Count Per Schema
--
-- VIEW
CREATE TABLE salix.in_nullable_attribute_count AS
SELECT
  na.TSchema,
  COUNT(*) AS Nb
FROM salix.in_nullable_attribute AS na
GROUP BY TSchema;

-- ALTER TABLE salix.in_nullable_attribute_count
--   ADD PRIMARY KEY (TSchema);

--
-- Attribute Count Per Schema
--
CREATE TABLE salix.in_attribute_count AS
SELECT
  c.table_schema AS TSchema,
  COUNT(*) AS Nb
FROM COLUMNS AS c
WHERE (c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas))
GROUP BY c.table_schema;

-- ALTER TABLE salix.in_attribute_count
--   ADD PRIMARY KEY (TSchema);

--
-- number of constraints per schema
-- argh! MySQL NOT NULL is missing: moved out altogether.
--
CREATE TABLE salix.in_constraint_count AS
SELECT
  s.schema_name AS TSchema,
  COUNT(tc.table_schema) AS Nb
FROM SCHEMATA AS s
-- may be zero...
LEFT JOIN TABLE_CONSTRAINTS AS tc
       ON (s.schema_name = tc.table_schema
       -- the condition must be in the join close for count(null)==0...
       AND tc.constraint_type <> 'CHECK') -- UNIQUE, PRIMARY KEY, FOREIGN KEY
WHERE (s.schema_name <> ALL (SELECT name FROM salix.excluded_schemas))
GROUP BY s.schema_name;

--
-- Nullable Attribute Rate Per Schema
--
-- VIEW
CREATE TABLE salix.in_nullable_attribute_rate AS
SELECT
  ac.TSchema,
  IfNULL((1.0 * nac.Nb / ac.Nb), 0.0) AS Rate          -- !! non standard
FROM salix.in_nullable_attribute_count AS nac
RIGHT JOIN salix.in_attribute_count AS ac
  ON (nac.TSchema=ac.TSchema);

-- ALTER TABLE salix.in_nullable_attribute_rate
--   ADD PRIMARY KEY (TSchema);

--
-- Nullable Attribute Rate in 20% .. 50%
--
-- VIEW
CREATE TABLE salix.da_nullable_attribute_rate_high AS
SELECT
  nar.TSchema,
  nar.Rate,
  CONCAT('schema ', nar.TSchema) AS explanation
FROM salix.in_nullable_attribute_rate AS nar
WHERE nar.Rate >= 0.20 AND nar.Rate < 0.50;


-- ALTER TABLE salix.da_nullable_attribute_rate_high
--   ADD PRIMARY KEY (TSchema);

--
-- Nullable Attribute Rate in 50% .. 80%
--
-- VIEW
CREATE TABLE salix.da_nullable_attribute_rate_quite_high AS
SELECT
  nar.TSchema,
  nar.Rate,
  CONCAT('schema ', nar.TSchema) AS explanation
FROM salix.in_nullable_attribute_rate AS nar
WHERE nar.Rate >= 0.50 AND nar.Rate < 0.80;

-- ALTER TABLE salix.da_nullable_attribute_rate_quite_high
--   ADD PRIMARY KEY (TSchema);

--
-- Nullable Attribute Rate over 80%
--
-- VIEW
CREATE TABLE salix.da_nullable_attribute_rate_too_high AS
SELECT
  nar.TSchema,
  nar.Rate,
  CONCAT('schema ', nar.TSchema) AS explanation
FROM salix.in_nullable_attribute_rate AS nar
WHERE nar.Rate >= 0.80;

-- ALTER TABLE salix.da_nullable_attribute_rate_too_high
--   ADD PRIMARY KEY (TSchema);

--
-- List of Foreign Key Constraints
--
-- VIEW
CREATE TABLE salix.in_f_key_constraint AS
SELECT
  -- ??? bug workaround...
  DISTINCT
  -- constraint
  tc.constraint_schema AS ConstraintSchema,
  tc.constraint_name AS ConstraintName,
  -- referencing table and attribute
  kcu.table_schema AS CSchema,
  kcu.table_name AS CTable,
  kcu.column_name AS CColumn,
  cc.data_type AS CType,
  cc.character_maximum_length AS CLength,
  cc.numeric_precision AS CPrecision,
  -- referenced table (foreign) and attribute
  ucu.table_schema AS FSchema,
  ucu.table_name AS FTable,
  ucu.column_name AS FColumn,
  fc.data_type AS FType,
  fc.character_maximum_length AS FLength,
  fc.numeric_precision AS FPrecision,
  -- build an explanation for the report
  CONCAT('constraint ', tc.constraint_schema, '.', tc.constraint_name,
  ' on attribute ', kcu.column_name, -- type?
  ' of table ', kcu.table_schema, '.', kcu.table_name,
  ' from attribute ', ucu.column_name, -- type?
  ' of table ', ucu.table_schema, '.', ucu.table_name) AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema
  AND tc.constraint_name=kcu.constraint_name
  AND tc.table_schema=kcu.table_schema
  AND tc.table_name=kcu.table_name)
JOIN COLUMNS AS cc
  ON (kcu.table_schema=cc.table_schema
  AND kcu.table_name=cc.table_name
  AND kcu.column_name=cc.column_name)
JOIN REFERENTIAL_CONSTRAINTS AS rc
  -- bug ??? note the constraint_name is necessarily NOT unique with Pg
  ON (tc.constraint_schema=rc.constraint_schema
  AND tc.constraint_name=rc.constraint_name)
JOIN KEY_COLUMN_USAGE AS ucu
  ON (rc.unique_constraint_schema=ucu.constraint_schema
  AND rc.unique_constraint_name=ucu.constraint_name
  -- this one is quite tricky, we must deal with composite foreign keys:
  -- FOREIGN KEY (a,b) REFERENCES (c,d)
  -- what ordinal_position and position_in_unique_constraint means depending
  -- on the type of key usage is not well documented.
  -- Well, in postgresql 8.3 it says "NOT IMPLEMENTED", although it is.
  AND kcu.position_in_unique_constraint = ucu.ordinal_position)
JOIN COLUMNS AS fc
  ON (ucu.table_schema=fc.table_schema
  AND ucu.table_name=fc.table_name
  AND ucu.column_name=fc.column_name)
WHERE tc.constraint_type = 'FOREIGN KEY'
  AND (tc.constraint_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- mysql: ERROR 1071 (42000):
-- Specified key was too long; max key length is 1000 bytes
-- ALTER TABLE salix.in_f_key_constraint
--   ADD PRIMARY KEY (ConstraintSchema,ConstraintName,
--                    CSchema,CTable,CColumn,
-- 		      FSchema,FTable,FColumn);

--
-- Foreign Key Types Don't Match
--
-- Seems impossible to create such a foreign key in MySQL ...
-- VIEW
CREATE TABLE salix.da_f_key_type_dont_match AS
SELECT *
FROM salix.in_f_key_constraint
WHERE CType <> FType;

-- mysql: ERROR 1071 (42000)
-- Specified key was too long; max key length is 1000 bytes
-- ALTER TABLE salix.da_f_key_type_dont_match
--   ADD PRIMARY KEY (ConstraintSchema,ConstraintName,CSchema,CTable,CColumn);

--
-- Foreign Key With Same Type But Different Lengths
--
-- VIEW
CREATE TABLE salix.da_f_key_match_b_diff_sizes AS
SELECT *
FROM salix.in_f_key_constraint
WHERE CType = FType
  AND (CLength <> FLength OR CPrecision <> FPrecision);

-- mysql: ERROR 1071 (42000)
-- Specified key was too long; max key length is 1000 bytes
-- ALTER TABLE salix.da_f_key_match_b_diff_sizes
--   ADD PRIMARY KEY (ConstraintSchema,ConstraintName,CSchema,CTable,CColumn);

--
-- Large Primary Keys Actually Referenced By A Foreign Key
--
-- VIEW
CREATE TABLE salix.da_large_p_keys_referenced_by_f_key AS
SELECT *
FROM salix.in_f_key_constraint
WHERE CType = FType
  AND ((CLength = FLength AND CLength > 16) OR
        -- ??? also check schema of type?
        (CType) = ANY (SELECT type_name FROM salix.not_fk_type));
-- mysql: ERROR 1071 (42000)
-- Specified key was too long; max key length is 1000 bytes
-- ALTER TABLE salix.da_large_p_keys_referenced_by_f_key
--   ADD PRIMARY KEY (ConstraintSchema,ConstraintName,CSchema,CTable,CColumn);

--
-- Foreign Keys Not Referencing A Primary Key
--
-- Seems impossible to create such a foreign key in MySQL ...
-- VIEW
CREATE TABLE salix.da_f_key_not_ref_p_key AS
SELECT
  tc.constraint_schema AS ConstraintSchema,
  tc.constraint_name AS ConstraintName,
  kcu.table_schema AS CSchema,
  kcu.table_name AS CTable,
  kcu.column_name AS CColumn,
  fkcu.table_schema AS FSchema,
  fkcu.table_name AS FTable,
  fkcu.column_name AS FColumn,
  ftc.constraint_schema AS FConstraintSchema,
  ftc.constraint_name AS FConstraintName,
  -- build an explanation for the report
  CONCAT('constraint ', tc.constraint_schema, '.', tc.constraint_name,
  ' on attribute ', kcu.column_name, -- type?
  ' of table ', kcu.table_schema, '.', kcu.table_name,
  ' from attribute ', fkcu.column_name, -- type?
  ' of table ', fkcu.table_schema, '.', fkcu.table_name) AS explanation
FROM TABLE_CONSTRAINTS AS tc
JOIN KEY_COLUMN_USAGE AS kcu
  ON (tc.constraint_schema=kcu.constraint_schema)
  AND (tc.constraint_name=kcu.constraint_name)
  AND (tc.table_schema=kcu.table_schema)
  AND (tc.table_name=kcu.table_name)
JOIN REFERENTIAL_CONSTRAINTS AS rc
  ON kcu.constraint_schema=rc.constraint_schema
  AND kcu.constraint_name=rc.constraint_name
JOIN KEY_COLUMN_USAGE AS fkcu
  ON rc.unique_constraint_schema=fkcu.constraint_schema
  AND rc.unique_constraint_name=fkcu.constraint_name
JOIN TABLE_CONSTRAINTS AS ftc
  ON (ftc.constraint_schema=fkcu.constraint_schema)
  AND (ftc.constraint_name=fkcu.constraint_name)
  AND (ftc.table_schema=fkcu.table_schema)
  AND (ftc.table_name=fkcu.table_name)
WHERE tc.constraint_type = 'FOREIGN KEY'
  AND NOT ftc.constraint_type = 'PRIMARY KEY'
  AND (tc.constraint_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- mysql: ERROR 1071 (42000)
-- Specified key was too long; max key length is 1000 bytes
-- ALTER TABLE salix.da_f_key_not_ref_p_key
--   ADD PRIMARY KEY (ConstraintSchema,ConstraintName,CSchema,CTable,CColumn);

--
-- Tables Which Are The Source From A Foreign Key
--
-- VIEW
CREATE TABLE salix.in_table_src_fkey_constraint AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  t.table_name AS TName
FROM TABLES AS t
JOIN TABLE_CONSTRAINTS AS tc
  ON t.table_schema=tc.table_schema AND t.table_name=tc.table_name
WHERE t.table_type = 'BASE TABLE'
  AND tc.constraint_type = 'FOREIGN KEY'
  AND (t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- ALTER TABLE salix.in_table_src_fkey_constraint
--   ADD PRIMARY KEY (TSchema,TName);

-- Symmetrical Version (useless but prettier)
/*
CREATE TABLE salix.in_table_src_fkey_constraint2 AS

SELECT DISTINCT
  t.table_schema AS TSchema,
  t.table_name AS TName
FROM TABLES AS t
  JOIN KEY_COLUMN_USAGE AS kcu
    ON  (t.table_schema=kcu.table_schema)
    AND (t.table_name=kcu.table_name)
  JOIN TABLE_CONSTRAINTS AS tc
    ON (kcu.constraint_schema=tc.constraint_schema)
    AND (kcu.constraint_name=tc.constraint_name)
    AND (kcu.table_schema=tc.table_schema)
    AND (kcu.table_name=tc.table_name)
WHERE tc.constraint_type = 'FOREIGN KEY'
  AND (t.table_schema <> 'information_schema')
  AND (t.table_schema <> 'salix')
  AND (t.table_schema <> 'mysql');
*/

--
-- Tables Which Are The Destination Of A Foreign Key
--
-- VIEW
CREATE TABLE salix.in_table_dst_fkey_constraint AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  t.table_name AS TName
FROM TABLES AS t
JOIN TABLE_CONSTRAINTS AS tc
  ON t.table_schema=tc.table_schema AND t.table_name=tc.table_name
JOIN REFERENTIAL_CONSTRAINTS AS rc
  ON tc.constraint_schema=rc.unique_constraint_schema
     AND tc.constraint_name=rc.unique_constraint_name
WHERE t.table_type = 'BASE TABLE'
  AND t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas);

-- ALTER TABLE salix.in_table_dst_fkey_constraint
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Schema w/o any FK
--
-- VIEWS
CREATE TABLE salix.in_table_count AS
SELECT
  t.table_schema AS TSchema,
  COUNT(*) AS Nb
FROM TABLES AS t
WHERE t.table_type = 'BASE TABLE'
  AND t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
GROUP BY t.table_schema;

-- ALTER TABLE salix.in_table_count
--   ADD PRIMARY KEY (TSchema);

--
-- Isolated Tables
--
-- VIEW
CREATE TABLE salix.da_isolated_table AS
SELECT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
JOIN salix.in_table_count AS tc ON (t.table_schema = tc.Tschema)
WHERE tc.Nb>1
  AND t.table_type = 'BASE TABLE'
  AND (t.table_schema, t.table_name) NOT IN
  -- EXCEPT is not implemented by MySQL
       (SELECT * FROM salix.in_table_src_fkey_constraint
        UNION
        SELECT * FROM salix.in_table_dst_fkey_constraint)
  -- redundant
  AND (t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

-- ALTER TABLE salix.da_isolated_table
--   ADD PRIMARY KEY (TSchema,TName);

-- count these table per schema
CREATE TABLE salix.in_isolated_table_count AS
SELECT
  it.TSchema,
  COUNT(*) AS Nb
FROM salix.da_isolated_table AS it
GROUP BY it.TSchema;

-- ALTER TABLE salix.in_isolated_table_count
--   ADD PRIMARY KEY (TSchema);

-- for MySQL, this is implied by the use of MyISAM?
CREATE TABLE salix.da_schema_wo_FK AS
SELECT tc.TSchema, CONCAT('schema ', tc.TSchema) AS explanation
FROM salix.in_table_count AS tc
JOIN salix.in_isolated_table_count AS itc ON tc.TSchema = itc.TSchema
WHERE tc.Nb = itc.Nb;

-- remove isolated tables if schema has no FK anyway
DELETE FROM salix.da_isolated_table
WHERE Tschema IN (SELECT Tschema FROM salix.da_schema_wo_FK);

-- ALTER TABLE salix.da_schema_wo_FK
--   ADD PRIMARY KEY (TSchema);

--
-- Table Identifiers Length <= 2
--
-- VIEW
CREATE TABLE salix.da_table_identifiers_length_too_short AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t -- tables, views...
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(t.table_name) <= 2;

-- ALTER TABLE salix.da_table_identifiers_length_too_short
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Table Identifiers Length = 3
--
-- VIEW
CREATE TABLE salix.da_table_identifiers_length_quite_short AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(t.table_name) = 3;

-- ALTER TABLE salix.da_table_identifiers_length_quite_short
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Table Identifiers Length = 4
--
-- VIEW
CREATE TABLE salix.da_table_identifiers_length_short AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(t.table_name) = 4;

-- ALTER TABLE salix.da_table_identifiers_length_short
--   ADD PRIMARY KEY (TSchema,TName);

--
-- Attribute Identifiers Length <= 1
--
-- VIEW
CREATE TABLE salix.da_attribute_identifiers_length_too_short AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  c.column_name AS CName,
  CONCAT('attribute ', c.column_name,
  ' of table ', c.table_schema, '.', c.table_name) AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(c.column_name) <= 1;

-- ALTER TABLE salix.da_attribute_identifiers_length_too_short
--   ADD PRIMARY KEY (TSchema,TName,CName);

--
-- Attribute Identifiers Length = 2
--
-- VIEW
CREATE TABLE salix.da_attribute_identifiers_length_quite_short AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  c.column_name AS CName,
  CONCAT('attribute ',c.column_name,
  ' of table ', c.table_schema, '.', c.table_name) AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(c.column_name) = 2
  -- both of these are okay?
  -- MySQL does not like "NOT IN ('id', 'pk')"
  AND LOWER(c.column_name) <> 'id'
  AND LOWER(c.column_name) <> 'pk';

-- ALTER TABLE salix.da_attribute_identifiers_length_quite_short
--   ADD PRIMARY KEY (TSchema,TName,CName);

--
-- Attribute Identifiers Length = 3
--
-- VIEW
CREATE TABLE salix.da_attribute_identifiers_length_short AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  c.column_name AS CName,
  CONCAT('attribute ', c.column_name,
    ' of table ', c.table_schema, '.', c.table_name) AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(c.column_name) = 3
  AND LOWER(c.column_name) NOT LIKE '%id%';

-- ALTER TABLE salix.da_attribute_identifiers_length_short
--   ADD PRIMARY KEY (TSchema,TName,CName);

--
-- Attribute Count Per Table
--
-- VIEW
CREATE TABLE salix.da_attribute_count_per_table_many AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  COUNT(*) AS Nb,
  CONCAT('table ', c.table_schema, '.', c.table_name,
   ' with ', salix.textcast(COUNT(*)), ' attributes') AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
GROUP BY c.table_schema, c.table_name
HAVING COUNT(*) BETWEEN 20 AND 40;

-- ALTER TABLE salix.da_attribute_count_per_table_many
--   ADD PRIMARY KEY (TSchema,TName);

CREATE TABLE salix.da_attribute_count_per_table_too_many AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  COUNT(*) AS Nb,
  CONCAT('table ', c.table_schema, '.', c.table_name,
   ' with ', salix.textcast(COUNT(*)), ' attributes') AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
GROUP BY c.table_schema, c.table_name
HAVING COUNT(*) > 40;

-- ALTER TABLE salix.da_attribute_count_per_table_too_many
--   ADD PRIMARY KEY (TSchema,TName);

--
-- homonymous heterogeneous attributes on tables (not views)
--
CREATE TABLE salix.da_attribute_same_name_diff_types AS
SELECT
  c.table_schema AS TSchema,
  c.column_name AS CName,
  COUNT(DISTINCT c.data_type) AS nb,
  CONCAT('attribute ',c.column_name,
   ' in schema ', c.table_schema,
   ' has ', salix.textcast(COUNT(DISTINCT c.data_type)), ' types')
    AS explanation
FROM TABLES AS t
JOIN COLUMNS AS c
  ON -- table_catalog...
     t.table_schema = c.table_schema AND
     t.table_name = c.table_name
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND t.table_type = 'BASE TABLE'
GROUP BY c.table_schema, c.column_name
HAVING COUNT(DISTINCT c.data_type)>1;

-- ALTER TABLE salix.da_attribute_same_name_diff_types
--   ADD PRIMARY KEY (TSchema,CName);

--
-- Schema Identifiers Length = 3
--
-- VIEW
CREATE TABLE salix.da_schema_identifiers_length_too_short AS
SELECT DISTINCT
  t.table_schema AS TSchema,
    CONCAT('schema ', t.table_schema) AS explanation
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(t.table_schema) <= 3;

--
-- Schema Identifiers Length = 4
--
-- VIEW
CREATE TABLE salix.da_schema_identifiers_length_quite_short AS
SELECT DISTINCT
  t.table_schema AS TSchema,
    CONCAT('schema ', t.table_schema) AS explanation
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(t.table_schema) = 4;

--
-- Attribute Identifiers Length = 5
--
-- VIEW
CREATE TABLE salix.da_schema_identifiers_length_short AS
SELECT DISTINCT
  t.table_schema AS TSchema,
    CONCAT('schema ', t.table_schema) AS explanation
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND CHAR_LENGTH(t.table_schema) = 5;

--
-- "10 ways to wreck your database" by Joshua Berkus
-- see http://www.youtube.com/watch?v=uFLRc6y_O3s
--

--
-- table name styles
--
CREATE TABLE salix.in_table_name_camel AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  COUNT(*) AS Nb
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(t.table_name, '[a-z][A-Z]')
GROUP BY t.table_schema;

CREATE TABLE salix.in_table_name_underline AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  COUNT(*) AS Nb
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(t.table_name, '_')
GROUP BY t.table_schema;

CREATE TABLE salix.in_table_name_capitalized AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  COUNT(*) AS Nb
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(t.table_name, '^[A-Z]')
GROUP BY t.table_schema;

CREATE TABLE salix.in_table_name_not_capitalized AS
SELECT DISTINCT
  t.table_schema AS TSchema,
  COUNT(*) AS Nb
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(t.table_name, '^[a-z]')
GROUP BY t.table_schema;

--
-- hmmm... what about singular vs plural? language dependent
--

CREATE TABLE salix.da_table_names_mixed_style AS
--
-- thisIsATable and_this_is_another
--
SELECT c.TSchema, 'names' AS Issue, NULL AS TName,
  CONCAT('schema ', c.TSchema) AS explanation
FROM salix.in_table_name_camel AS c
JOIN salix.in_table_name_underline AS u ON (c.TSchema=u.TSchema)
UNION
--
-- ThisTable thisother
--
SELECT c.TSchema, 'caps' AS Issue, NULL AS TName,
  CONCAT('schema ', c.TSchema) AS explanation
FROM salix.in_table_name_capitalized AS c
JOIN salix.in_table_name_not_capitalized AS n ON (c.TSchema=n.TSchema)
UNION
--
-- tblStuff
--
SELECT t.table_schema AS TSchema, 'hungarian' AS Issue, t.table_name AS TName,
  CONCAT('hungarian table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
WHERE t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND (salix.regexpr(t.table_name, '^[Tt][Bb][Ll]')
    OR salix.regexpr(t.table_name, '[Tt][Bb][Ll]$')
    OR salix.regexpr(t.table_name, '^[Tt][Aa][Bb][Ll][Ee]')
    OR salix.regexpr(t.table_name, '[Tt][Aa][Bb][Ll][Ee]$'));

--
-- Attribute name styles...
--

CREATE TABLE salix.in_attribute_name_camel AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  COUNT(*) AS Nb
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(c.column_name, '[a-z][A-Z]')
GROUP BY c.table_schema, c.table_name;

CREATE TABLE salix.in_attribute_name_underline AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  COUNT(*) AS Nb
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(c.column_name, '_')
GROUP BY c.table_schema, c.table_name;

CREATE TABLE salix.in_attribute_name_capitalized AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  COUNT(*) AS Nb
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(c.column_name, '^[A-Z]')
GROUP BY c.table_schema, c.table_name;

CREATE TABLE salix.in_attribute_name_not_capitalized AS
SELECT DISTINCT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  COUNT(*) AS Nb
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(c.column_name, '^[a-z]')
GROUP BY c.table_schema, c.table_name;

CREATE TABLE salix.da_attribute_names_mixed_style AS
--
-- hisAttribute this_attribute
--
SELECT c.TSchema, 'names' AS Issue, c.TName AS TName,
  CONCAT('table ', c.TSchema, '.', c.TName) AS explanation
FROM salix.in_attribute_name_camel AS c
JOIN salix.in_attribute_name_underline AS u
  ON (c.TSchema=u.TSchema AND c.TName=u.TName)
UNION
--
-- ThisAttribute thisOther
--
SELECT c.TSchema, 'caps' AS Issue, c.TName AS TName,
  CONCAT('table ', c.TSchema, '.', c.TName) AS explanation
FROM salix.in_attribute_name_capitalized AS c
JOIN salix.in_attribute_name_not_capitalized AS n
  ON (c.TSchema=n.TSchema AND c.TName=n.TName);

--
-- attributes which are names as their containing table
--
CREATE TABLE salix.da_attribute_with_table_prefix AS
SELECT
  c.table_schema AS TSchema,
  c.table_name AS TName,
  c.column_name AS CName,
  CONCAT('attribute ', c.column_name,
    ' of table ', c.table_schema, '.', c.table_name) AS explanation
FROM COLUMNS AS c
WHERE c.table_schema <> ALL (SELECT name FROM salix.excluded_schemas)
  AND salix.regexpr(c.column_name, CONCAT('^', c.table_name, '.*'));

-- ALTER TABLE salix.da_attribute_same_name_diff_types
--   ADD PRIMARY KEY (TSchema,TName,CName);
