-- $Id: adv_system.sql 2788 2011-01-28 14:00:04Z coelho $

--
-- MAATKIT http://www.maatkit.org/
-- mk-duplicate-key-checker
--  - Find duplicate foreign keys on MySQL tables.
--

-- number of columns of a FK constraint
CREATE TABLE salix.in_fk_nb_cols AS
SELECT
  rc.constraint_schema,
  rc.constraint_name,
  COUNT(*) AS nbcol,
  -- by the way, it is about:
  cu.table_schema,
  cu.table_name
FROM REFERENTIAL_CONSTRAINTS AS rc
JOIN KEY_COLUMN_USAGE AS cu
  ON (rc.constraint_schema = cu.constraint_schema AND
      rc.constraint_name = cu.constraint_name)
GROUP BY
  rc.constraint_schema, rc.constraint_name,
  cu.table_schema, cu.table_name;

-- number of common columns of distinct KF constraints
CREATE TABLE salix.in_fk_nb_common_cols AS
SELECT DISTINCT
  rc1.constraint_schema AS cs1,
  rc1.constraint_name AS cn1,
  rc2.constraint_schema AS cs2,
  rc2.constraint_name AS cn2,
  COUNT(*) AS nbcol
FROM REFERENTIAL_CONSTRAINTS AS rc1
JOIN KEY_COLUMN_USAGE AS kcu1
  ON (rc1.constraint_schema = kcu1.constraint_schema AND
      rc1.constraint_name = kcu1.constraint_name)
JOIN KEY_COLUMN_USAGE AS kcu2
  ON (kcu2.table_schema = kcu1.table_schema AND
      kcu2.table_name = kcu1.table_name AND
      kcu2.column_name = kcu1.column_name)
JOIN REFERENTIAL_CONSTRAINTS AS rc2
  ON (rc2.constraint_schema = kcu2.constraint_schema AND
      rc2.constraint_name = kcu2.constraint_name)
WHERE rc1.constraint_schema<>rc2.constraint_schema
   OR rc1.constraint_name<>rc2.constraint_name
GROUP BY rc1.constraint_schema, rc1.constraint_name,
  rc2.constraint_schema, rc2.constraint_name;

-- 2 -> 2*1, 1 redundant
-- 3 -> 3*2, 2 redundants
-- 4 -> 4*3, 3 redundants
-- 5 -> 5*4, 4 redundants...
-- when we reach the maximum, we have a redundancy.
-- beware that an FK may be *INCLUDED* into another, must check both...
CREATE TABLE salix.sa_redundant_foreign_keys AS
SELECT
  i1.table_schema AS TSchema,
  i1.table_name AS TName,
  FLOOR(SQRT(COUNT(*))) AS Nb,
  CONCAT('table ', i1.table_schema, '.', i1.table_name) AS explanation
FROM salix.in_fk_nb_common_cols AS c
JOIN salix.in_fk_nb_cols AS i1
  ON (c.cs1 = i1.constraint_schema AND
      c.cn1 = i1.constraint_name AND
      c.nbcol = i1.nbcol)
JOIN salix.in_fk_nb_cols AS i2
  ON (c.cs2 = i2.constraint_schema AND
      c.cn2 = i2.constraint_name AND
      c.nbcol = i2.nbcol)
GROUP BY TSchema, TName;
