-- $Id: adv_dbversion.sql 2923 2012-01-07 09:58:29Z coelho $

-- ------------------------------------------------------------
-- Database Version Advices
-- ------------------------------------------------------------

CREATE TABLE salix.in_current_database_version AS
SELECT *
FROM salix.in_db_versions AS dbv
WHERE salix.software_info() RLIKE dbv.info_re -- !! non standard ?
ORDER BY priority ASC
LIMIT 1;

--
-- MySQL Warning
--
-- VIEW
CREATE TABLE salix.dbva_database_is_mysql AS
SELECT cdv.*, s.schema_name, CONCAT('schema ', s.schema_name) AS explanation
FROM salix.in_current_database_version AS cdv, -- one row
     salix_information_schema.SCHEMATA AS s
WHERE (s.schema_name <> ALL (SELECT name FROM salix.excluded_schemas))
  AND cdv.name LIKE '%MySQL%';

--
-- backend engines
--
CREATE TABLE salix.mysql_table_engine(
  eid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  engine VARCHAR(50) NOT NULL,
  ename VARCHAR(50) NOT NULL,
  safe BOOLEAN NOT NULL, -- crash-safe and ACID
  PRIMARY KEY (eid),
  UNIQUE KEY (ename),
  UNIQUE KEY (engine)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO salix.mysql_table_engine(engine,ename,safe) VALUES
('MyISAM', 'MyISAM', FALSE), -- the default engine
('BDB', 'Berkeley DB', TRUE), -- discontinued in 5.1
('ISAM', 'ISAM', FALSE), -- discontinued in 5.0
('InnoDB', 'InnoDB', TRUE), -- InnoBase Oy engine (owned by Oracle since 2005)
('Maria', 'Maria', TRUE),
('Falcon', 'Falcon', TRUE),
('MEMORY', 'Memory', FALSE), -- also named HEAP ?
('IBMDB2I', 'IBM DB2 i', TRUE),
('MRG_MYISAM', 'Merge', FALSE), -- MyISAM
('FEDERATED', 'Federated', FALSE), -- link...
('ARCHIVE', 'Archive', FALSE),
('CSV', 'CSV', FALSE), -- just a flat file
('BLACKHOLE', 'Black Hole', FALSE); -- joke: discard data engine...

--
-- MyISAM Warning On Table
--
-- VIEW
CREATE TABLE salix.dbva_table_engine_is_unsafe AS
SELECT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
WHERE
  -- non standard attribute. if unknown, considered unsafe
      (t.engine <> ALL (SELECT engine FROM salix.mysql_table_engine WHERE safe))
  AND (t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

--
-- MyISAM Warning On Schema
--
-- VIEW
CREATE TABLE salix.dbva_schema_engine_is_unsafe AS
SELECT
  teim.TSchema AS TSchema,
  CONCAT('schema ', teim.TSchema) AS explanation
FROM salix.dbva_table_engine_is_unsafe AS teim
GROUP BY teim.TSchema;

--
-- Engine inconsistency
--
-- VIEW
CREATE TABLE salix.in_table_engine_is_safe AS
SELECT
  t.table_schema AS TSchema,
  t.table_name AS TName,
  CONCAT('table ', t.table_schema, '.', t.table_name) AS explanation
FROM TABLES AS t
WHERE (t.engine = ANY (SELECT engine FROM salix.mysql_table_engine WHERE safe))
  AND (t.table_schema <> ALL (SELECT name FROM salix.excluded_schemas));

CREATE TABLE salix.dbva_engine_inconsistency AS
SELECT
  teim.TSchema AS TSchema,
  CONCAT('schema ', teim.TSchema) AS explanation
FROM salix.dbva_table_engine_is_unsafe AS teim
JOIN salix.in_table_engine_is_safe AS teii
  ON teim.TSchema=teii.TSchema
GROUP BY teim.TSchema;

--
-- Table & Index in different schemas
--
-- (seems impossible to create?)
-- VIEW
CREATE TABLE salix.ca_cross_schema_indexes AS
SELECT DISTINCT
  s.index_schema AS IndexSchema,
  s.index_name AS IndexName,
  s.table_schema AS TableSchema,
  s.table_name AS TableName,
  CONCAT('index ', s.index_schema, '.', s.index_name,
  ' on table ', s.table_schema, '.', s.table_name) AS explanation
FROM STATISTICS AS s
WHERE s.index_schema <> s.table_schema
  AND s.index_schema <> ALL (SELECT name FROM salix.excluded_schemas);
