-- $Id: infrastructure.sql 2923 2012-01-07 09:58:29Z coelho $

-- --------------------------------------------
-- Salix Babylonica Infrastructure Tables & Procedures
-- --------------------------------------------

--
-- Excluded Schemas List
--
CREATE TABLE salix.excluded_schemas(
  tid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  name varchar(50) NOT NULL,
  PRIMARY KEY (tid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO salix.excluded_schemas(name)
VALUES
	('mysql'),
	('information_schema'),
	('salix'),
	('salix_information_schema'),
	('test_design'),
	('test_consistency'),
	('test_myisam_fk');

-- types not to be used as FK
CREATE TABLE salix.not_fk_type(
  fid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  schema_name VARCHAR(50) NOT NULL DEFAULT '', -- ???
  type_name VARCHAR(50) NOT NULL,
  PRIMARY KEY (fid),
  UNIQUE KEY (schema_name, type_name)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO salix.not_fk_type(type_name) VALUES
('TEXT'),
('BLOB'),
('TINYTEXT'),
('MEDIUMTEXT'),
('MEDIUMBLOB'),
('LONGTEXT'),
('LONGBLOB');

--
-- Only Tables List
--
/* Not working (issue with the case where the table doesn't exist / contains NULL)
CREATE TABLE salix.only_tables(
	tid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
	name varchar(50),
	PRIMARY KEY (tid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO salix.only_tables(
	name
) VALUES
	('%%');
*/

--
-- Advice Category
--

CREATE TABLE salix.advice_category(
  cid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  description varchar(50) NOT NULL,
  PRIMARY KEY (cid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

--
-- Advice Severity
--
CREATE TABLE salix.advice_severity(
  sid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  description varchar(50) NOT NULL,
  PRIMARY KEY (sid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

--
-- Advice Level
--
CREATE TABLE salix.advice_level(
  lid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  description varchar(20) UNIQUE NOT NULL,
  PRIMARY KEY (lid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

--
-- Advice Info
--
CREATE TABLE salix.advice_info(
  aid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  view_name varchar(50) NOT NULL,
  cid mediumint(8) unsigned NOT NULL,
  sid mediumint(8) unsigned NOT NULL,
  lid mediumint(8) unsigned NOT NULL,
  active boolean NOT NULL DEFAULT TRUE,
  title varchar(100) NOT NULL,
  abstract text NOT NULL,
  description text,
  PRIMARY KEY (aid),
  KEY FK_advice_info_category (cid),
  KEY FK_advice_info_severity (sid),
  CONSTRAINT unique_view_name UNIQUE (view_name),
  CONSTRAINT FK_advice_info_category FOREIGN KEY (cid) REFERENCES advice_category (cid),
  CONSTRAINT FK_advice_info_severity
     FOREIGN KEY (sid) REFERENCES advice_severity (sid),
  CONSTRAINT FK_advice_info_level FOREIGN KEY (lid) REFERENCES advice_level(lid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

--
-- advised PK types
--
CREATE TABLE salix.primary_key_types
(
  pktid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  -- type catalog? type schema?
  type_name VARCHAR(64) NOT NULL,
  PRIMARY KEY (pktid),
  CONSTRAINT type_once UNIQUE (type_name)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO salix.primary_key_types(type_name)
VALUES
  ('int'),
  ('tinyint'),
  ('smallint'),
  ('mediumint'),
  ('bigint')
;

--
-- Known Versions and Comments
--
-- add Warning number ?
-- (e.g. 0, no warning ; 1, not up to date ;
--       2, security update ; 3, test version ... etc.)
CREATE TABLE salix.in_db_versions (
  did smallint NOT NULL AUTO_INCREMENT,
  name varchar(45) NOT NULL,
  info_re varchar(45) NOT NULL,
  sid mediumint(8) unsigned NOT NULL,
  comments varchar(200),
  priority smallint(5) NOT NULL,
  PRIMARY KEY (did),
  KEY FK_db_versions_severity (sid),
  CONSTRAINT FK_db_versions_severity
    FOREIGN KEY (sid) REFERENCES advice_severity (sid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
