-- $Id: standard_information_schema.sql 2923 2012-01-07 09:58:29Z coelho $
-- build something that looks like the real information_schema
-- from what is provided by MySQL...

CREATE TABLE CHARACTER_SETS AS
SELECT *
FROM information_schema.CHARACTER_SETS;

CREATE TABLE COLLATIONS AS
SELECT *
FROM information_schema.COLLATIONS;

CREATE TABLE COLLATION_CHARACTER_SET_APPLICABILITY AS
SELECT *
FROM information_schema.COLLATION_CHARACTER_SET_APPLICABILITY;

CREATE TABLE COLUMNS AS
SELECT *
FROM information_schema.COLUMNS;
CREATE INDEX index_table_schema ON COLUMNS (table_schema);
CREATE INDEX index_table_name ON COLUMNS (table_name);
CREATE INDEX index_column_name ON COLUMNS (column_name);

CREATE TABLE COLUMN_PRIVILEGES AS
SELECT *
FROM information_schema.COLUMN_PRIVILEGES;

-- sorry, temporary tables have additionnal use restrictions...
CREATE TABLE KEY_COLUMN_USAGE_COPY AS
SELECT constraint_catalog,
       constraint_schema,
       -- this is necessary, because constraint_name
       -- is often non unique (say 'PRIMARY')
       CONCAT(table_name,'/',constraint_name) AS constraint_name,
       table_catalog,
       table_schema,
       table_name,
       column_name,
       ordinal_position,
       position_in_unique_constraint,
       referenced_table_schema,
       referenced_table_name,
       referenced_column_name
FROM information_schema.KEY_COLUMN_USAGE;
CREATE INDEX index_constraint_schema ON KEY_COLUMN_USAGE_COPY (constraint_schema);
CREATE INDEX index_constraint_name ON KEY_COLUMN_USAGE_COPY (constraint_name);
CREATE INDEX index_table_schema ON KEY_COLUMN_USAGE_COPY (table_schema);
CREATE INDEX index_table_name ON KEY_COLUMN_USAGE_COPY (table_name);
CREATE INDEX index_column_name ON KEY_COLUMN_USAGE_COPY (column_name);
CREATE INDEX index_referenced_table_schema ON KEY_COLUMN_USAGE_COPY (referenced_table_schema);
CREATE INDEX index_referenced_table_name ON KEY_COLUMN_USAGE_COPY (referenced_table_name);
CREATE INDEX index_referenced_column_name ON KEY_COLUMN_USAGE_COPY (referenced_column_name);

-- build standard column usage
CREATE TABLE KEY_COLUMN_USAGE AS
SELECT constraint_catalog,
       constraint_schema,
       constraint_name,
       table_catalog,
       table_schema,
       table_name,
       column_name,
       ordinal_position,
       position_in_unique_constraint
FROM KEY_COLUMN_USAGE_COPY;
CREATE INDEX index_constraint_schema ON KEY_COLUMN_USAGE (constraint_schema);
CREATE INDEX index_constraint_name ON KEY_COLUMN_USAGE (constraint_name);
CREATE INDEX index_table_schema ON KEY_COLUMN_USAGE (table_schema);
CREATE INDEX index_table_name ON KEY_COLUMN_USAGE (table_name);
CREATE INDEX index_column_name ON KEY_COLUMN_USAGE (column_name);
-- CREATE INDEX index_referenced_table_schema ON KEY_COLUMN_USAGE (referenced_table_schema);
-- CREATE INDEX index_referenced_table_name ON KEY_COLUMN_USAGE (referenced_table_name);
-- CREATE INDEX index_referenced_column_name ON KEY_COLUMN_USAGE (referenced_column_name);

-- not always there?
-- CREATE TABLE PROFILING AS
-- SELECT *
-- FROM information_schema.PROFILING;

CREATE TABLE ROUTINES AS
SELECT *
FROM information_schema.ROUTINES;

CREATE TABLE SCHEMATA AS
SELECT *
FROM information_schema.SCHEMATA;

CREATE TABLE SCHEMA_PRIVILEGES AS
SELECT *
FROM information_schema.SCHEMA_PRIVILEGES;

CREATE TABLE STATISTICS AS
SELECT *
FROM information_schema.STATISTICS;

CREATE TABLE TABLES AS
SELECT *
FROM information_schema.TABLES;
CREATE INDEX index_table_schema ON TABLES (table_schema);
CREATE INDEX index_table_name ON TABLES (table_name);

CREATE TABLE TABLE_CONSTRAINTS AS
SELECT
  constraint_catalog,
  constraint_schema,
  -- this is necessary, because constraint_name
  -- is often non unique (say 'PRIMARY')
  CONCAT(table_name,'/',constraint_name) AS constraint_name,
  NULL AS table_catalog,
  table_schema,
  table_name,
  constraint_type,
  'NO' AS is_deferrable,
  'NO' AS initially_deferred
FROM information_schema.TABLE_CONSTRAINTS;
CREATE INDEX index_constraint_schema ON TABLE_CONSTRAINTS (constraint_schema);
CREATE INDEX index_constraint_name ON TABLE_CONSTRAINTS (constraint_name);
CREATE INDEX index_table_schema ON TABLE_CONSTRAINTS (table_schema);
CREATE INDEX index_table_name ON TABLE_CONSTRAINTS (table_name);

CREATE TABLE TABLE_PRIVILEGES AS
SELECT *
FROM information_schema.TABLE_PRIVILEGES;

CREATE TABLE TRIGGERS AS
SELECT *
FROM information_schema.TRIGGERS;

CREATE TABLE USER_PRIVILEGES AS
SELECT *
FROM information_schema.USER_PRIVILEGES;

CREATE TABLE VIEWS AS
SELECT *
FROM information_schema.VIEWS;

-- should be temporary but...
CREATE TABLE KEY_COLUMN_USAGE_COUNT AS
SELECT -- constraint_catalog, -- is NULL
       constraint_schema,
       constraint_name,
       COUNT(*) AS columns_nb
FROM KEY_COLUMN_USAGE_COPY
GROUP BY constraint_schema, constraint_name;

-- reconstruct the "true" standard referential_constraints relation
-- based on MySQL collapsing of the information in key_column_usage.
-- unfortunately, MySQL does not tell which unique constraint is referenced:
-- in the unlikely case where several are available on a subset of attributes,
-- we return all of them, which may lead to incorrect counts in later queries.
--
-- NOTE: this now exists in MySQL 5.1.10
-- moreover the unique bug above is fixed since 5.1.16

CREATE TABLE REFERENTIAL_CONSTRAINTS AS
SELECT
  constraint_catalog,
  constraint_schema,
  CONCAT(table_name, '/', constraint_name) AS constraint_name,
  unique_constraint_catalog,
  unique_constraint_schema,
  CONCAT(referenced_table_name, '/', unique_constraint_name)
     AS unique_constraint_name,
  match_option,
  update_rule,
  delete_rule
FROM information_schema.REFERENTIAL_CONSTRAINTS;

-- SELECT
--   tc1.constraint_catalog,
--   tc1.constraint_schema,
--   kcuc1.constraint_name  AS constraint_name,
--   tc2.constraint_catalog AS unique_constraint_catalog,
--   tc2.constraint_schema AS unique_constraint_schema,
--   kcuc2.constraint_name  AS unique_constraint_name,
--   'NONE' AS match_option,
--   'NO ACTION' AS update_rule,
--   'NO ACTION' AS delete_rule
-- FROM TABLE_CONSTRAINTS AS tc1
--     -- source of foreign key
-- JOIN KEY_COLUMN_USAGE_COPY AS kcu1
-- 		ON -- constraint_catalog is NULL in MySQL
-- 		    tc1.constraint_schema = kcu1.constraint_schema
-- 		AND tc1.constraint_name = kcu1.constraint_name
-- 		   -- table_catalog is NULL in MySQL
-- 		AND tc1.table_schema = kcu1.table_schema
-- 		AND tc1.table_name = kcu1.table_name
-- JOIN KEY_COLUMN_USAGE_COUNT AS kcuc1
--        ON -- catalog...
--            kcuc1.constraint_schema = kcu1.constraint_schema
--        AND kcuc1.constraint_name = kcu1.constraint_name
-- 	-- destination of foreign key
-- 	JOIN KEY_COLUMN_USAGE_COPY AS kcu2
-- 		ON kcu1.referenced_table_schema = kcu2.table_schema
-- 		AND kcu1.referenced_table_name = kcu2.table_name
-- 		AND kcu1.referenced_column_name =  kcu2.column_name
-- 		AND kcu1.position_in_unique_constraint = kcu2.ordinal_position
-- 	JOIN TABLE_CONSTRAINTS AS tc2
-- 		ON -- constraint_catalog is NULL in MySQL
-- 		    tc2.constraint_schema = kcu2.constraint_schema
-- 		AND tc2.constraint_name = kcu2.constraint_name
-- 		   -- table_catalog is NULL in MySQL
-- 		AND tc2.table_schema = kcu2.table_schema
-- 		AND tc2.table_name = kcu2.table_name
-- 	JOIN KEY_COLUMN_USAGE_COUNT AS kcuc2
--        ON kcuc2.constraint_schema = kcu2.constraint_schema
--        AND kcuc2.constraint_name = kcu2.constraint_name
-- WHERE tc1.constraint_type = 'FOREIGN KEY'
--   AND (tc2.constraint_type = 'PRIMARY KEY' OR tc2.constraint_type = 'UNIQUE')
--   AND kcuc1.columns_nb = kcuc2.columns_nb;

-- clean up should be 'TEMPORARY' tables...
DROP TABLE KEY_COLUMN_USAGE_COUNT;
DROP TABLE KEY_COLUMN_USAGE_COPY;
