-- $Id: infrastructure.sql 2923 2012-01-07 09:58:29Z coelho $

-- --------------------------------------------
-- Salix Babylonica Infrastructure Tables & Procedures
-- --------------------------------------------

--
-- Excluded Tables List
-- schema names that are ignored
--
CREATE TABLE salix.excluded_schemas
( tid SERIAL PRIMARY KEY,
  name TEXT UNIQUE NOT NULL
);

INSERT INTO salix.excluded_schemas(name)
VALUES	('pg_catalog'),
	('pg_toast'),
	('pg_toast_temp_1'),
	('information_schema'),
	('salix_information_schema'),
--	('public'), -- should I?
	('salix');

--
-- "large" types not to be used as FK
--
CREATE TABLE salix.not_fk_type
( fid SERIAL PRIMARY KEY,
  -- mmh...
  schema_name TEXT NOT NULL DEFAULT 'pg_catalog',
  type_name TEXT NOT NULL,
  UNIQUE (schema_name, type_name)
);

INSERT INTO salix.not_fk_type(type_name)
VALUES ('bytea'),
       ('text'),
       ('name'),
--       ('numeric'), -- there is an associated max size/precision...
       ('path'),
       ('polygon');

--
-- Only Tables List
--
/* Not working
 * (issue with the case where the table doesn't exist / contains NULL)
CREATE TABLE salix.only_tables(
	tid mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
	name varchar(50),
	PRIMARY KEY (tid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO salix.only_tables(
	name
) VALUES
	('%%');
*/

--
-- Advice Category
--
CREATE TABLE salix.advice_category
( cid SERIAL PRIMARY KEY,
  description TEXT UNIQUE NOT NULL
);

--
-- Advice Severity
--
CREATE TABLE salix.advice_severity
( sid SERIAL PRIMARY KEY,
  description TEXT UNIQUE NOT NULL
);

--
-- Advice Level
--
CREATE TABLE salix.advice_level
( lid SERIAL PRIMARY KEY,
  description TEXT UNIQUE NOT NULL
);

--
-- Advice Info and Description
--
CREATE TABLE salix.advice_info
( aid SERIAL PRIMARY KEY,
  view_name TEXT UNIQUE NOT NULL,
  cid INTEGER NOT NULL REFERENCES salix.advice_category,
  sid INTEGER NOT NULL REFERENCES salix.advice_severity,
  lid INTEGER NOT NULL REFERENCES salix.advice_level,
  active BOOLEAN NOT NULL DEFAULT TRUE,
  -- maybe the title should be unique?
  -- or possibly at a given severity?
  title TEXT NOT NULL,
  abstract TEXT NOT NULL,
  description TEXT
);

--
-- primary key types that seem advisable.
--
CREATE TABLE salix.primary_key_types
(
  pktid SERIAL PRIMARY KEY,
  -- type catalog ? type schema ?
  type_name TEXT UNIQUE NOT NULL
);

COPY salix.primary_key_types(type_name) FROM STDIN;
integer
smallint
bigint
oid
\.


--
-- Known Versions and Comments
--
-- add Warning number ? (e.g. 0, no warning ; 1, not up to date ; 2, security update ; 3, test version ... etc)
CREATE TABLE salix.in_db_versions (
  did SERIAL PRIMARY KEY,
  name varchar(45) NOT NULL,
  info_re varchar(45) NOT NULL,
  sid integer NOT NULL references advice_severity,
  comments varchar(200),
  priority integer NOT NULL
);
