#! /bin/sh
#
# $Id: salix.sh 3073 2012-01-21 10:36:38Z coelho $
#

# generated version description
VDESCR='@VDESCR@'

# default directory for Salix SQL scripts
DEFAULT_SLXBBL_HOME='/usr/local/share/slxbbl'

# use default if home not set
[ "$SLXBBL_HOME" ] || SLXBBL_HOME="$DEFAULT_SLXBBL_HOME"

# show command usage
usage()
{
  status=$1 msg="$2"
  shift 2
  {
    [ "$msg" ] && { echo "ERROR: $msg" ; echo ; }
    echo "Salix usage:"
    echo "  $0 (create|show|report|drop) (my|pg) <database connection options>"
    echo "  see 'man salix' for more help"
    echo
    echo "Salix version $VDESCR"
    echo "  See http://coelho.net/salix/ for the latest version"
    echo
    echo "(c) 2008-@YEAR@" \
	 "Alexandre Aillos, Samuel Pilot, Shamil Valeev, Fabien Coelho"
    echo
    echo "This software is distributed under the term of the GPLv3."
    echo "Go to http://www.gnu.org/copyleft/gpl.html for details"
  } >&2
  exit $status
}

# process options
case $1 in
  version|--version|-v)
    echo "$0 version $VDESCR"
    exit 0
    ;;
  help|--help|-h)
    usage 0 ''
    ;;
esac

# get the two arguments
[ $# -lt 2 ] && usage 1 'at least 2 arguments are expected'

op=$1 db=$2
shift 2

# possibly silently exchange arguments
# so as to be compatible with previous order
if [ "$db" = 'create' -o "$db" = 'show' -o "$db" = 'report' -o "$db" = 'drop' ]
then
  op=$db db=$op
fi

# check operation
case $op in
  create|show|report|drop)
    ;;
  *)
    usage 2 "unexpected operation '$op'"
    ;;
esac

# check database
case $db in
  # supported databases
  my|mysql)
    db=mysql cmd=mysql
    ;;
  pg|pgsql|postgres|postgresql|psql)
    db=pgsql cmd=psql
    ;;
  # unsupported databases
  or|ora|oracle|sqlplus)
    db=oracle cmd=sqlplus
    usage 3 "Oracle support not implemented yet"
    ;;
  ms|mssql|ms-sql|mssqlserver|ms-sql-server|sqlserver|sql-server)
    usage 3 "MS SQL Server support not implemented yet"
    ;;
  sqlite)
    usage 3 "SQLite support cannot be implemented, no information_schema"
    ;;
  # catch other cases as errors
  *)
    usage 3 "unexpected database argument '$db'"
    ;;
esac

# SQL script name for operation on database
file="${db}_${op}.sql"

# check SQL script directory
[ -d "$SLXBBL_HOME" ] ||
  usage 4 "salix home not found at '$SLXBBL_HOME'"

# changing directory is necessary because of relative includes
cd "$SLXBBL_HOME" ||
  usage 5 "cannot cd to '$SLXBBL_HOME'"

# check script
test -f "$file" ||
  usage 6 "operation '$op' not implemented for $db (no '$file')"

# run!
exec $cmd "$@" < "$file"
